@@ -29,6 +29,7 @@
 #include <glib.h>
 
 typedef enum {
+	CD_MEDIA_TYPE_BUSY,
 	CD_MEDIA_TYPE_ERROR,
 	CD_MEDIA_TYPE_UNKNOWN,
 	CD_MEDIA_TYPE_CD,
@@ -55,6 +56,11 @@ typedef enum {
 	CDDRIVE_TYPE_DVD_DRIVE              = 1 << 8,
 } CDDriveType;
 
+typedef enum {
+	CD_PROTOCOL_IDE,
+	CD_PROTOCOL_SCSI,
+} CDProtocolType;
+
 typedef struct {
 	CDDriveType type;
 	char *display_name;
@@ -62,6 +68,7 @@ typedef struct {
 	int max_speed_read;
 	char *cdrecord_id;
 	char *device;
+	CDProtocolType protocol;
 } CDDrive;
 
 /* Returns a list of CDDrive structs */
