@@ -19,7 +19,6 @@
 #include <libgnome/gnome-i18n.h>
 
 #include "cd-recorder.h"
-#include "cdrecorder-marshal.h"
 
 #undef DEBUG_CDRECORD
 
@@ -49,6 +48,7 @@ struct CDRecorderPrivate {
 	gboolean send_return;
 	gboolean expect_cdrecord_to_die;
 	GList *actions;
+	gboolean dangerous;
 
 	char *last_error;
 	
@@ -61,7 +61,6 @@ enum {
 	PROGRESS_CHANGED,
 	ACTION_CHANGED,
 	ANIMATION_CHANGED,
-	CANCEL_FUNC_CHANGED,
 	INSERT_CD_REQUEST,
 	INSERT_RWCD_REQUEST,
 	LAST_SIGNAL
@@ -109,10 +108,44 @@ cd_recorder_get_type (void)
 	return cd_recorder_type;
 }
 
+void
+cd_recorder_track_free (Track *track)
+{
+	switch (track->type) {
+	case TRACK_TYPE_DATA:
+		g_free (track->contents.data.filename);
+		break;
+	case TRACK_TYPE_AUDIO:
+		g_free (track->contents.audio.filename);
+		g_free (track->contents.audio.cdtext);
+		break;
+	default:
+		g_warning ("Invalid track type %d", track->type);
+	}
+}
+
 static gboolean
-cd_needs_growisofs (CDDrive *recorder)
+can_burn_dvds (CDDrive *recorder)
+{
+	if (!(recorder->type & CDDRIVE_TYPE_DVD_RW_RECORDER) &&
+	    !(recorder->type & CDDRIVE_TYPE_DVD_PLUS_R_RECORDER) &&
+	    !(recorder->type & CDDRIVE_TYPE_DVD_PLUS_RW_RECORDER)) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+cd_needs_growisofs (CDDrive *recorder, GList *tracks)
 {
 	CDMediaType type;
+	GList *l;
+
+	/* If we cannot burn DVDs, then we don't need growisofs */
+	if (can_burn_dvds (recorder) == FALSE) {
+		return FALSE;
+	}
 
 	type = guess_media_type (recorder->device);
 	if (type == CD_MEDIA_TYPE_DVDR
@@ -122,18 +155,30 @@ cd_needs_growisofs (CDDrive *recorder)
 		return TRUE;
 	}
 
+	/* If we have audio tracks, we're using cdrecord */
+	for (l = tracks; l != NULL; l = l->next) {
+		Track *track = l->data;
+		if (track->type == TRACK_TYPE_AUDIO) {
+			return FALSE;
+		}
+	}
+
 	return FALSE;
 }
 
-static void
-cancel_cdrecord (gpointer data)
+gboolean
+cd_recorder_cancel (CDRecorder *cdrecorder, gboolean skip_if_dangerous)
 {
-	CDRecorder *cdrecorder = (CDRecorder *) data;
+	if (!cdrecorder->priv->dangerous || !skip_if_dangerous) {
+		kill (cdrecorder->priv->pid, SIGINT);
 
-	kill (cdrecorder->priv->pid, SIGINT);
+		cdrecorder->priv->result = RESULT_CANCEL;
+		g_main_loop_quit (cdrecorder->priv->loop);
 
-	cdrecorder->priv->result = RESULT_CANCEL;
-	g_main_loop_quit (cdrecorder->priv->loop);
+		return FALSE;
+	}
+
+	return cdrecorder->priv->dangerous;
 }
 
 void
@@ -194,9 +239,7 @@ growisofs_stdout_read (GIOChannel   *source,
 				       cd_recorder_table_signals[PROGRESS_CHANGED], 0,
 				       percent);
 		} else if (strstr (line, "About to execute") != NULL) {
-			g_signal_emit (G_OBJECT (cdrecorder),
-				       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-				       cancel_cdrecord, TRUE);
+			cdrecorder->priv->dangerous = TRUE;
 		}
 
 		g_free (line);
@@ -272,9 +315,7 @@ cdrecord_stdout_read (GIOChannel   *source,
 				       1.0);
 			cdrecorder->priv->result = RESULT_FINISHED;
 		} else if (g_str_has_prefix (line, "Last chance to quit, ")) {
-			g_signal_emit (G_OBJECT (cdrecorder),
-				       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-				       cancel_cdrecord, TRUE);
+			cdrecorder->priv->dangerous = TRUE;
 		} else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[ACTION_CHANGED], 0,
@@ -423,9 +464,7 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 	cdrecorder->priv->tracks = tracks;
 	cdrecorder->priv->track_count = g_list_length (tracks);
 
-	/* TODO: this could be changed so that cd_needs_growisofs()
-	   takes tracks, and will decide cdrecord/growisofs/error. */
-	if (cd_needs_growisofs (recorder)) {
+	if (cd_needs_growisofs (recorder, tracks)) {
 		return cd_recorder_write_growisofs (cdrecorder,
 						    recorder, tracks, speed,
 						    eject, blank, dummy_write);
@@ -508,15 +547,13 @@ retry_growisofs:
 	g_signal_emit (G_OBJECT (cdrecorder),
 			cd_recorder_table_signals[ANIMATION_CHANGED], 0,
 			TRUE);
-	g_signal_emit (G_OBJECT (cdrecorder),
-			cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-			cancel_cdrecord, FALSE);
+	cdrecorder->priv->dangerous = FALSE;
 
 #ifdef DEBUG_CDRECORD
 	{
-		int i;
+		guint i;
 		g_print ("launching command: ");
-		for (i = 0; argv->len; i++) {
+		for (i = 0; i < argv->len; i++) {
 			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
 		}
 		g_print ("\n");
@@ -534,9 +571,9 @@ retry_growisofs:
 					&stdout_pipe,
 					&stderr_pipe,
 					&error)) {
-		int i;
+		guint i;
 		g_warning ("growisofs command failed: %s\n", error->message);
-		for (i = 0; argv->len; i++) {
+		for (i = 0; i < argv->len; i++) {
 			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
 		}
 		g_print ("\n");
@@ -564,9 +601,7 @@ retry_growisofs:
 					     cdrecorder);
 		g_io_channel_unref (channel);
 
-		g_signal_emit (G_OBJECT (cdrecorder),
-			       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-			       cancel_cdrecord, FALSE);
+		cdrecorder->priv->dangerous = FALSE;
 
 		g_main_loop_run (cdrecorder->priv->loop);
 		g_main_loop_unref (cdrecorder->priv->loop);
@@ -587,6 +622,14 @@ retry_growisofs:
 			cd_recorder_table_signals[ANIMATION_CHANGED], 0,
 			FALSE);
 
+	if (eject != FALSE && cdrecorder->priv->result == RESULT_FINISHED) {
+		char *cmd;
+
+		cmd = g_strdup_printf ("eject %s", recorder->device);
+		g_spawn_command_line_sync (cmd, NULL, NULL, NULL, NULL);
+		g_free (cmd);
+	}
+
 	return cdrecorder->priv->result;
 }
 
@@ -677,9 +720,7 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 	g_signal_emit (G_OBJECT (cdrecorder),
 		       cd_recorder_table_signals[ANIMATION_CHANGED], 0,
 		       TRUE);
-	g_signal_emit (G_OBJECT (cdrecorder),
-		       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-		       cancel_cdrecord, FALSE);
+	cdrecorder->priv->dangerous = FALSE;
 
 #ifdef DEBUG_CDRECORD
 	{
@@ -689,7 +730,6 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 			g_print ("%s ", (char*)g_ptr_array_index (argv, i));
 		}
 		g_print ("\n");
-		return -1;
 	}
 #endif
 
@@ -728,10 +768,8 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 					     cdrecord_stderr_read,
 					     cdrecorder);
 		g_io_channel_unref (channel);
-		
-		g_signal_emit (G_OBJECT (cdrecorder),
-			       cd_recorder_table_signals[CANCEL_FUNC_CHANGED], 0,
-			       cancel_cdrecord, FALSE);
+
+		cdrecorder->priv->dangerous = FALSE;
 
 		g_main_loop_run (cdrecorder->priv->loop);
 		g_main_loop_unref (cdrecorder->priv->loop);
@@ -848,15 +886,6 @@ cd_recorder_class_init (CDRecorderClass *klass)
 				NULL, NULL,
 				g_cclosure_marshal_VOID__BOOLEAN,
 				G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-	cd_recorder_table_signals[CANCEL_FUNC_CHANGED] =
-		g_signal_new ("cancel-func-changed",
-				G_TYPE_FROM_CLASS (klass),
-				G_SIGNAL_RUN_LAST,
-				G_STRUCT_OFFSET (CDRecorderClass,
-					cancel_func_changed),
-				NULL, NULL,
-				cdrecorder_marshal_VOID__POINTER_BOOLEAN,
-				G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_BOOLEAN);
 	cd_recorder_table_signals[INSERT_CD_REQUEST] =
 		g_signal_new ("insert-cd-request",
 				G_TYPE_FROM_CLASS (klass),
