@@ -39,6 +39,8 @@
 #include "nautilus-cd-burner.h"
 #include "make-iso.h"
 
+#define MAX_ISO_NAME_LEN 32
+
 enum {
 	CANCEL_NONE,
 	CANCEL_MAKE_ISO,
@@ -553,82 +555,36 @@ display_error_dialog (GtkWindow *win, const gchar *msg)
 	gtk_widget_destroy (error_dlg);
 }
 
-static void
-file_chooser_response_cb (GtkWidget *chooser, gint response, gpointer data)
-{
-	gchar *file_name;
-	char **file_name_out;
-
-	file_name_out = data;
-
-	if (response != GTK_RESPONSE_OK) {
-		if (response != GTK_RESPONSE_DELETE_EVENT) {
-			gtk_widget_destroy (chooser);
-		}
-		gtk_main_quit ();
-
-		return;
-	}
-
-	file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
-
-	if (!file_name)
-		return;
-
-	if (strlen (file_name) < 1) {
-		g_free (file_name);
-		return;
-	}
-
-	/* Change into directory if that's what user selected */
-	if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
-		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-						     file_name);
-		g_free (file_name);
-	} else {
-		if (g_file_test (file_name, G_FILE_TEST_EXISTS)) {
-			if (!overwrite_existing_file (GTK_WINDOW (chooser), file_name)) {
-				g_free (file_name);
-				return;
-			}
-		}
-
-		gtk_widget_destroy (GTK_WIDGET (chooser));
-
-		g_free (*file_name_out);
-		*file_name_out = file_name;
-
-		gtk_main_quit ();
-	}
-}
-
 static char *
 select_iso_filename (void)
 {
 	GtkWidget *chooser;
 	char *filename;
-
-	chooser = gtk_file_chooser_dialog_new (_("Choose a filename for the cdrom image"),
-					       NULL,
-					       GTK_FILE_CHOOSER_ACTION_SAVE,
-					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					       GTK_STOCK_OK, GTK_RESPONSE_OK,
-					       NULL);
+	gint response;
+	chooser = gtk_file_chooser_dialog_new_with_backend (_("Choose a filename for the cdrom image"),
+							    NULL,
+							    GTK_FILE_CHOOSER_ACTION_SAVE,
+							    "unix",
+							    GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+							    GTK_STOCK_OK, GTK_RESPONSE_OK,
+							    NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (chooser),
 					   "image.iso");
-
+ reselect:
 	filename = NULL;
-	g_signal_connect (G_OBJECT (chooser),
-			  "response",
-			  G_CALLBACK(file_chooser_response_cb),
-			  &filename);
+	response = gtk_dialog_run (GTK_DIALOG (chooser));
+	
+	if (response == GTK_RESPONSE_OK) {
+		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+		if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+			if (!overwrite_existing_file (GTK_WINDOW (chooser), filename)) {
+				g_free (filename);
+				goto reselect;
+			}
+		}
 
-	gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
-
-	gtk_widget_show (chooser);
-
-	/* Run in a loop untill we get the response from file selector */
-	gtk_main ();
+	}
+	gtk_widget_destroy (chooser);
 
 	return filename;
 }
@@ -878,6 +834,28 @@ target_changed (GtkOptionMenu *option_menu, gpointer data)
   refresh_dialog (xml);
 }
 
+static void
+cdname_entry_insert_text (GtkEditable *editable,
+			  const gchar *new_text,
+			  gint         new_text_length,
+			  gint        *position)
+{
+	char *current_text;
+
+	if (new_text_length < 0) {
+		new_text_length = strlen (new_text);
+	}
+
+	current_text = gtk_editable_get_chars (editable, 0, -1);
+	if (strlen (current_text) +  new_text_length >= MAX_ISO_NAME_LEN) {
+		gdk_display_beep (gtk_widget_get_display (GTK_WIDGET (editable)));
+		g_signal_stop_emission_by_name (editable, "insert_text");
+	}
+		
+	g_free (current_text);
+}
+
+
 static void
 init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean from_iso)
 {
@@ -890,7 +868,7 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	GtkWidget *entry;
 	GtkWidget *reuse;
 	int size_mb;
-	char *str;
+	char *str, *last;
 	char buf[129];
 	GDate *now;
 
@@ -923,6 +901,8 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 	
 	/* Set default name */
 	entry = glade_xml_get_widget (xml, "cdname_entry");
+	g_signal_connect (entry, "insert_text", G_CALLBACK (cdname_entry_insert_text),
+			  NULL);
 	if (from_iso != FALSE) {
 		gtk_widget_set_sensitive (entry, FALSE);
 		if (disc_name != NULL) {
@@ -941,6 +921,18 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 		g_date_strftime (buf, sizeof (buf), _("Personal Data, %b %d, %Y"), now);
 
 		g_date_free (now);
+
+		/* Cut off at 32 bytes */
+		last = str = buf;
+		while (*str != 0 &&
+		       (str - buf) < MAX_ISO_NAME_LEN) {
+			last = str;
+			str = g_utf8_next_char (str);
+		}
+		if (*str != 0) {
+			*last = 0;
+		}
+		
 		gtk_entry_set_text (GTK_ENTRY (entry), buf);
 	}
 
